PROGRAM PLY_View;

{
Dit programma leest de records van een *.PLY file en toont de data
op het scherm of schrijft de data in leesbare vorm naar schijf.
Het programma moet aangeroepen worden vanuit de directory waarin
de *.PLY file staat.
Dit programma is een bewerking van LAYOTOOL.PAS uit de LAYO1 Toolbox.

G.A. te Riet Scholten,
Universiteit Twente, Instrumentendienst.
}

USES  CRT;

CONST
{  max_data = 2046; }
   max_data = 30000;
   LF       = #10;
   CR       = #13;
   Bell     = #7;
   Pauze    = 25;

TYPE
  String100 = STRING[100];
  FileNaam  = STRING[12];

  wrd_array = ARRAY[0..max_data] OF WORD;
  int_array = ARRAY[0..max_data] OF INTEGER;

VAR
  block : ^wrd_array;
  attr  : ^wrd_array;
  xpos  : ^int_array;
  ypos  : ^int_array;
  net   : ^wrd_array;

  top_array : word;

  board_offset_x : INTEGER;
  board_offset_y : INTEGER;
  board_size_x   : INTEGER;
  board_size_y   : INTEGER;

  ToDisk    : BOOLEAN;
  OutF      : TEXT;
  Proj_name : FileNaam;


PROCEDURE Smooth;
BEGIN
    IF NOT ToDisk THEN Delay(Pauze);
END;


PROCEDURE Init;
BEGIN
  Write('Toewijzen van heapruimte ....');   
  NEW(block); fillchar(block^,sizeof(block^),0);
  NEW(attr);  fillchar(attr^ ,sizeof(attr^) ,0);
  NEW(xpos);  fillchar(xpos^ ,sizeof(xpos^) ,0);
  NEW(ypos);  fillchar(ypos^ ,sizeof(ypos^) ,0);
  NEW(net);   fillchar(net^  ,sizeof(net^)  ,0);
  top_array := 0;
  Write(CR); ClrEol;
END;


FUNCTION Load_Ply(F_Name:FileNaam):BOOLEAN;
TYPE  Ply_Rec = RECORD
                  blk : WORD;
                  att : WORD;
                  xps : INTEGER;
                  yps : INTEGER;
                  net : WORD; 
                END;

VAR   I, F1_Len : WORD;
      F1        : FILE OF Ply_Rec;
      F1_rec    : Ply_Rec;
      OK        : BOOLEAN;

BEGIN
  Load_PLY:=FALSE;
  ASSIGN(F1,F_Name);
  {$I-} RESET(F1) {$I+};
  OK:=(IoResult = 0);
  IF NOT OK THEN
  BEGIN
    IF ToDisk THEN WriteLn('!! ',F_Name,' niet gevonden.');
    WriteLn(OutF,'!! ',F_Name,' niet gevonden.');
    EXIT;
  END;
  F1_Len := FileSize(F1);
  IF F1_Len > Max_Data THEN
  BEGIN
    WriteLn(OutF,'!! File te groot : ',Max_Data:5,' datarecords gereserveerd,');
    WriteLn(OutF,'!!          voor   ',F1_Len:5,' datarecords in ',F_Name,'.PLY');
    EXIT;
  END;
  Load_PLY:=TRUE;

  Init;
  
  FOR i := 1 TO F1_Len DO
  BEGIN
    READ(F1,F1_Rec);
    block^[i] := F1_Rec.blk;
    attr^[i]  := F1_Rec.att;
    xpos^[i]  := F1_Rec.xps;
    ypos^[i]  := F1_Rec.yps;
    net^[i]   := F1_Rec.net;
  END;
  CLOSE(F1);

  Top_Array      := F1_Len;
  board_offset_x := xpos^[2] + 8;
  board_offset_y := ypos^[2] + 8;
  board_size_x   := (xpos^[8] - xpos^[2]) - 16;
  board_size_y   := (ypos^[8] - ypos^[2]) - 16;
END;


PROCEDURE Initialisation;
BEGIN
  ToDisk:=FALSE;
  ClrScr;
  WriteLn(LF,LF,LF,'PLY_View versie 1.0');
  WriteLn('  *.PLY en *.CNF dump utility',LF);
  Write  ('- Naam van .PLY en .CNF file (zonder extensie) : ');
  ReadLn(Proj_Name);
  WriteLn;
  Write  ('- Uitvoer naar Scherm of hardDisk [S, D] ? ');
  IF Upcase(ReadKey)='D' THEN ToDisk:=TRUE; WriteLn;
  IF ToDisk THEN ASSIGN(OutF,Proj_Name+'.ASC')
            ELSE AssignCRT(OutF);
  {$I-} REWRITE(OUTF); {$I+}
  IF IOResult<>0
  THEN BEGIN WriteLn('!! Kan ',Proj_Name,'.ASC niet aanmaken!',Bell); HALT END;
  IF NOT Load_Ply(Proj_Name+'.PLY') THEN HALT;
END;


PROCEDURE Show_Data;
VAR  i, Blk, Atr : WORD;
     Kar         : CHAR;
BEGIN
  IF ToDisk
    THEN Write(CR,LF,'      van ',Top_Array,' datarecords geschreven',CR)
    ELSE ClrScr;
  WriteLn(OutF,'=================== ',Proj_Name,
               '.PLY =======================================');
  WriteLn(OutF,' Line Block Pad Tool Layer Pen Rot(Deg)',
               ' XMirr YMirr Net   XPos   YPos');
  IF NOT ToDisk THEN WINDOW(1,3,80,24);
  FOR I := 1 TO Top_Array DO
  BEGIN
    IF ToDisk THEN Write(I:5,CR);
    Blk:=Block^[i];
    Atr:=Attr^[i];
    Write(OutF,I:5);
    
    IF Blk AND $1FFF > 0 THEN Write(OutF,Blk AND $1FFF:6)               {Block}
                         ELSE Write(OutF,'      ');
    IF Atr AND $380  > 0
    THEN
    BEGIN
      IF Atr AND $80 = $80
        THEN Write(OutF,Atr AND $78 SHR 3:4,                              {Pad}
                       (Blk SHR 10)+(Atr AND 7):5,'          ')          {Tool}
        ELSE IF Atr AND $100 = $100                            {Pad# on Layer1}
               THEN Write(OutF,Atr AND $7F:4,'        1      ')
               ELSE IF Atr AND $200 = $200                     {Pad# on Layer2}
                      THEN Write(OutF,Atr AND $7F:4,'        2      ')
                      ELSE Write(OutF,'                   ');
                                          
      Write(OutF,(Atr SHR 10 AND $3F)*7.5 :9:1);                     {Rotation}
      Write(OutF,Blk SHR 15:6,Blk SHR 14 AND 1:6);           {XMirror, YMirror}
    END
    ELSE
      Write(OutF,'           ',Atr AND $78 SHR 3:2,Atr AND 7:6,   {Layer & Pen}
                 '                     ');
    IF Net^[i] AND $1FFF > 0 THEN Write(OutF,Net^[i] AND $1FFF:4)
                             ELSE Write(OutF,'    ');
    WriteLn(OutF,XPos^[I]:7,YPos^[I]:7);
    Smooth;
  END;
  IF ToDisk THEN BEGIN
                   ClrEol; 
                   WriteLn('Data in ',Proj_Name,'.asc.')
                 END
            ELSE BEGIN 
                   Write('Sla een toets aan om te vervolgen ....');
                   WHILE NOT KeyPressed DO {NOTHING};
                   Kar:=ReadKey; WriteLn(LF);
                 END;
END;


PROCEDURE Show_Cnf_File;
VAR
  F1              : TEXT;
  w1              : String100;
  i, Max_Pad_Read : WORD;

BEGIN
  ASSIGN(F1,Proj_Name+'.CNF');
  {$I-} RESET(F1);  {$i+}
  IF IOResult > 0 THEN
  BEGIN
    WriteLn(OutF,Proj_Name,'.CNF not found...');
    EXIT;
  END;
  IF NOT ToDisk THEN WINDOW(1,1,80,2);
  WriteLn(OutF,'============== ',Proj_Name,'.CNF =================');
  IF NOT ToDisk THEN BEGIN ClrEol; WINDOW(1,3,80,24); ClrScr; END;
  ReadLn(F1,w1); WriteLn(OutF,'Version : ',w1);
  Max_Pad_Read := 15;
  IF POS('4.85',w1) > 0 THEN Max_Pad_Read := 127;
  ReadLn(F1,w1); WriteLn(OutF,'Program name   : ',w1); Smooth;
  ReadLn(F1,w1); WriteLn(OutF,'File name      : ',w1); Smooth;
  ReadLn(F1,w1); WriteLn(OutF); Smooth;
  ReadLn(F1,w1); WriteLn(OutF); Smooth; 
  ReadLn(F1,w1); WriteLn(OutF,'Board size x   : ',w1); Smooth;
  ReadLn(F1,w1); WriteLn(OutF,'Board size y   : ',w1); Smooth; 
  ReadLn(F1,w1); WriteLn(OutF,'Board offset x : ',w1); Smooth;
  ReadLn(F1,w1); WriteLn(OutF,'Board offset y : ',w1); Smooth;
  ReadLn(F1,w1); WriteLn(OutF,'LAY file name  : ',w1); Smooth;
  ReadLn(F1,w1); WriteLn(OutF,'PLY file name  : ',w1); Smooth;
  ReadLn(F1,w1); WriteLn(OutF,'CMP file name  : ',w1); Smooth;
  ReadLn(F1,w1); WriteLn(OutF,'NET file name  : ',w1); Smooth;

  FOR i:=1 TO 6 DO
  BEGIN
    ReadLn(F1,w1);
    WriteLn(OutF); Smooth;
  END;

  FOR i:=0 TO 15 DO
  BEGIN
    ReadLn(F1,w1);
    WriteLn(OutF,'Tool_Diam[',i,'] : ',w1); Smooth;
  END;
  ReadLn(F1,w1);
  WriteLn(OutF,'Pad_Type       x-      y-     x+    y+    corner');
  FOR i:=0 TO Max_Pad_Read DO
  BEGIN
    ReadLn(F1,w1);
    WriteLn(OutF,COPY(w1,1,8),   { pad type  }
                 COPY(w1,9,8),   { x1 }
                 COPY(w1,17,8),  { y1 }
                 COPY(w1,25,8),  { x2 }
                 COPY(w1,33,8),  { y2 }
                 COPY(w1,41,8)); { corner }
    Smooth;
  END;
  ReadLn(F1,w1);
  FOR i:=1 TO 7 DO
  BEGIN
    ReadLn(F1,w1);
    WriteLn(OutF,'Pen_Diam[',i,'] = ',w1);
    Smooth;
  END;
  ReadLn(F1,w1);
  WHILE NOT EOF(F1) DO
  BEGIN
    ReadLn(F1,w1);
    IF w1 > '' THEN WriteLn(OutF,w1);
    Smooth;
  END;
  WriteLn(OutF,'============== ',Proj_Name,'.CNF END =============');
  CLOSE(F1);
END;


BEGIN
  Initialisation;
  Show_Data;
  Show_Cnf_File;
  CLOSE(OutF);
  WriteLn(LF,'Klaar'); 
END.

